import { constants } from "../constants";
import { AxoSupportedPlatforms } from "../types";
import { btModulesLoadConfig } from "../lib/safeLoadBtModule/types";
import { safeLoadBtModule } from "../lib/safeLoadBtModule";
import { loadScript } from "../lib/script-utils";
import loadAxo from "../loadAxoScript";

jest.mock("../lib/safeLoadBtModule");
const mockSafeLoadBtModule = safeLoadBtModule as jest.MockedFunction<
  typeof safeLoadBtModule
>;

jest.mock("../lib/script-utils");
const mockLoadScript = loadScript as jest.MockedFn<typeof loadScript>;

describe("loadAxoScript", () => {
  let testContext;
  const version = "3.109.0";
  const originalWindow = window;
  const originalPerformance = performance;
  const originalGetElementsByTagName = document.getElementsByTagName;
  let mockFastlane;

  beforeEach(() => {
    mockFastlane = { create: jest.fn() };

    testContext = {
      mockLoadResponse: {
        fastlane: mockFastlane,
        metadata: {
          localeUrl: "https://www.paypalobjects.com/connect-boba/locales/",
        },
      },
      client: {
        getVersion: () => {
          return version;
        },
      },
    };

    mockLoadScript.mockImplementation(async () => {
      window.braintree = window.braintree || {};
      window.braintree.loadFastlane = jest.fn().mockResolvedValue(mockFastlane);
      return window.braintree.loadFastlane;
    });

    // Mock performance API
    Object.defineProperty(globalThis, "performance", {
      value: {
        mark: jest.fn(),
        measure: jest.fn(),
      },
    });
  });

  afterEach(() => {
    Object.defineProperty(globalThis, "window", {
      value: originalWindow,
    });

    Object.defineProperty(globalThis, "performance", {
      value: originalPerformance,
    });

    Object.defineProperty(document, "getElementsByTagName", {
      value: originalGetElementsByTagName,
    });
  });

  describe("Unhappy path", () => {
    it("rejects promise if platform is invalid", async () => {
      const promise = loadAxo({
        // @ts-expect-error for test purposes
        platform: "Invalid Platform",
        btSdkVersion: version,
        minified: true,
      });
      await expect(promise).rejects.toThrowError(
        "unsupported Fastlane platform"
      );
    });

    it("rejects promise if load-fastlane broke the API contract", async () => {
      mockLoadScript.mockImplementation(async () => {
        return Promise.resolve(mockFastlane);
      });

      const promise = loadAxo({
        platform: AxoSupportedPlatforms.BT,
        btSdkVersion: version,
        minified: true,
      });

      await expect(promise).rejects.toThrowError(
        `Cannot read properties of undefined (reading 'loadFastlane')`
      );
    });

    it("rejects promise if loadScript rejects", async () => {
      const mockErrorMsg = "having a bad day";
      mockLoadScript.mockRejectedValueOnce(new Error(mockErrorMsg));

      const promise = loadAxo({
        platform: AxoSupportedPlatforms.BT,
        btSdkVersion: version,
        minified: true,
      });

      await expect(promise).rejects.toThrowError(mockErrorMsg);
    });
  });

  it("should load Bt hcf and axo when platform is bt", async () => {
    await loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: true,
    });

    expect(mockSafeLoadBtModule).toHaveBeenCalledTimes(1);
    expect(mockSafeLoadBtModule).toHaveBeenCalledWith(
      btModulesLoadConfig.hostedFields,
      version,
      true // minified
    );
  });

  it("should load Bt client, hcf and Axo when platform is ppcp", async () => {
    await loadAxo({
      platform: AxoSupportedPlatforms.PPCP,
      btSdkVersion: version,
      minified: true,
    });

    expect(mockSafeLoadBtModule).toHaveBeenCalledTimes(2);

    expect(mockSafeLoadBtModule).toHaveBeenCalledWith(
      btModulesLoadConfig.client,
      version,
      true // minified
    );
    expect(mockSafeLoadBtModule).toHaveBeenCalledWith(
      btModulesLoadConfig.hostedFields,
      version,
      true // minified
    );
  });

  it("should call loadScript with the minified URL when specified", async () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: true,
    });

    await expect(promise).resolves.toStrictEqual(testContext.mockLoadResponse);
  });

  it("should load the unminified URL when specified", async () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: false,
    });

    await expect(promise).resolves.toStrictEqual(testContext.mockLoadResponse);
    expect(mockLoadScript).toHaveBeenCalled();
    expect(mockLoadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_ASSET_PATH}/${constants.LOAD_FASTLANE_SCRIPT.unminified}.js`,
      })
    );
  });

  it("should default to the minified script when no environment is specified", async () => {
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
    });

    await expect(promise).resolves.toStrictEqual(testContext.mockLoadResponse);
    expect(mockLoadScript).toHaveBeenCalled();
    expect(mockLoadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `${constants.CDNX_PROD}/${constants.AXO_ASSET_PATH}/${constants.LOAD_FASTLANE_SCRIPT.minified}.js`,
      })
    );
  });

  it("should load the bundle override URL when specified", async () => {
    const bundleId = "2bc4343c92655";
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      metadata: { bundleIdOverride: bundleId },
    });

    await expect(promise).resolves.toStrictEqual({
      fastlane: mockFastlane,
      metadata: {
        localeUrl: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.LOCALE_PATH}`,
      },
    });
    expect(mockLoadScript).toHaveBeenCalled();
    expect(mockLoadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.AXO_ASSET_PATH}/${constants.LOAD_FASTLANE_SCRIPT.minified}.js`,
      })
    );
  });

  it("should call loadScript with the minified URL and bundle override URL when specified", async () => {
    const bundleId = "9df9369c92655";
    const promise = loadAxo({
      platform: AxoSupportedPlatforms.BT,
      btSdkVersion: version,
      minified: true,
      metadata: { bundleIdOverride: bundleId },
    });

    await expect(promise).resolves.toStrictEqual({
      fastlane: mockFastlane,
      metadata: {
        localeUrl: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.LOCALE_PATH}`,
      },
    });
    expect(mockLoadScript).toHaveBeenCalled();
    expect(mockLoadScript).toHaveBeenCalledWith(
      expect.objectContaining({
        src: `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${constants.AXO_ASSET_PATH}/${constants.LOAD_FASTLANE_SCRIPT.minified}.js`,
      })
    );
  });
});
