import { constants } from "./constants";
import {
  LoadAxoOptions,
  GenerateAssetUrlParams,
  AxoSupportedPlatforms,
} from "./types";
import { safeLoadBtModule } from "./lib/safeLoadBtModule";
import { btModulesLoadConfig } from "./lib/safeLoadBtModule/types";
import { loadScript } from "./lib/script-utils";

/**
 * Loads accelerated checkout components.
 * @param options object with a minified parameter to determine if the script that is loaded should be minified or not (defaults to true if)
 * @returns an object with metadata with a localeUrl parameter to be read by AXO SDK
 */
async function loadAxo(options: LoadAxoOptions) {
  performance.mark("pp_axo_sdk_init_invoked");
  const { btSdkVersion, minified } = options;
  const localeUrl = getLocaleUrl(options);

  if (options.platform === AxoSupportedPlatforms.BT) {
    await Promise.all([
      safeLoadBtModule(
        btModulesLoadConfig.hostedFields,
        btSdkVersion,
        minified
      ),
    ]);
  } else if (options.platform === AxoSupportedPlatforms.PPCP) {
    await Promise.all([
      safeLoadBtModule(btModulesLoadConfig.client, btSdkVersion, minified),
      safeLoadBtModule(
        btModulesLoadConfig.hostedFields,
        btSdkVersion,
        minified
      ),
    ]);
  } else {
    throw new Error("unsupported Fastlane platform");
  }

  await loadScript({
    id: "load-fastlane",
    src: getAssetPath(options),
    name: "loadFastlane",
    exportedName: "braintree.loadFastlane",
  });

  // if window.braintree.loadFastlane is undefined loadScript above with throw
  // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
  const fastlane = await window.braintree.loadFastlane!({
    unminified: !minified,
  });
  delete window.braintree.loadFastlane;

  return { fastlane, metadata: { localeUrl } };
}

/**
 * Prepends the domain to the asset url
 * @param options object with assetUrl and bundleid parameters to determine which URL to return
 * @returns full domain and assets URL as string
 */
function generateAssetUrl({
  assetPath,
  bundleId,
}: GenerateAssetUrlParams): string {
  return bundleId
    ? `https://cdn-${bundleId}.static.engineering.dev.paypalinc.com/${assetPath}`
    : `${constants.CDNX_PROD}/${assetPath}`;
}

/**
 * Retrieves either the minified or unminified assets URL as specified
 * @param options (Optional) object with a minified and metadata with bundleIdOverride parameters to determine which URL to return
 * @returns assets URL as string
 */
function getAssetPath(options?: LoadAxoOptions): string {
  const assetName =
    options?.minified !== false
      ? constants.LOAD_FASTLANE_SCRIPT.minified
      : constants.LOAD_FASTLANE_SCRIPT.unminified;

  const assetPath = `${constants.AXO_ASSET_PATH}/${assetName}.js`;

  return generateAssetUrl({
    assetPath,
    bundleId: options?.metadata?.bundleIdOverride,
  });
}

/**
 * Retrieves the Locales URL, the path to our language files
 * @param options (Optional) object with a minified and metadata with bundleIdOverride parameters to determine which URL to return
 * @returns locale URL as string
 */
function getLocaleUrl(options?: LoadAxoOptions): string {
  return generateAssetUrl({
    assetPath: constants.LOCALE_PATH,
    bundleId: options?.metadata?.bundleIdOverride,
  });
}

export default loadAxo;
